# Copyright (c) 2016 b<>com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.utils.translation import gettext_lazy as _
import horizon.exceptions
import horizon.messages
import horizon.tables


class StrategiesFilterAction(horizon.tables.FilterAction):
    # server = choices query = text
    filter_type = "server"
    filter_choices = (
        ('goal', _("Goal ="), True),
    )
    policy_rules = (("infra-optim", "strategy:detail"),)


class StrategiesTable(horizon.tables.DataTable):

    uuid = horizon.tables.Column(
        'uuid',
        verbose_name=_("UUID"),
        link="horizon:admin:strategies:detail")

    name = horizon.tables.Column(
        'name',
        verbose_name=_('Name'))

    display_name = horizon.tables.Column(
        'display_name',
        verbose_name=_('Verbose Name'))

    goal = horizon.tables.Column(
        'goal_name',
        verbose_name=_("Goal"),
    )

    def get_object_id(self, datum):
        return datum.uuid

    class Meta(object):
        name = "strategies"
        verbose_name = _("Strategies")
        table_actions = (
            StrategiesFilterAction,
        )


class RelatedStrategiesTable(horizon.tables.DataTable):

    uuid = horizon.tables.Column(
        'uuid',
        verbose_name=_("UUID"),
        link="horizon:admin:strategies:detail")

    name = horizon.tables.Column(
        'name',
        verbose_name=_('Name'))

    display_name = horizon.tables.Column(
        'display_name',
        verbose_name=_('Verbose Name'))

    goal = horizon.tables.Column(
        'goal_name',
        verbose_name=_("Goal"),
    )

    def get_object_id(self, datum):
        return datum.uuid

    class Meta(object):
        name = "related_strategies"
        verbose_name = _("Related strategies")
        hidden_title = False



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from horizon.test.settings import *  # noqa
from openstack_dashboard.test.settings import *  # noqa

# Load the pluggable dashboard settings
import openstack_dashboard.enabled
import watcher_dashboard.local.enabled

INSTALLED_APPS = (
    'django.contrib.contenttypes',
    'django.contrib.auth',
    'django.contrib.sessions',
    'django.contrib.staticfiles',
    'django.contrib.messages',
    'django.contrib.humanize',
    'openstack_auth',
    'compressor',
    'horizon',
    'openstack_dashboard',
)

INSTALLED_APPS = list(INSTALLED_APPS)  # Make sure it's mutable
settings_utils.update_dashboards(
    [
        watcher_dashboard.local.enabled,
        openstack_dashboard.enabled,
    ],
    HORIZON_CONFIG,
    INSTALLED_APPS,
)



# Copyright (c) 2016 b<>com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

from django.utils.translation import gettext_lazy as _
import horizon.exceptions
import horizon.tables
import horizon.tabs
from horizon.utils import memoized
import horizon.workflows

from watcher_dashboard.api import watcher
from watcher_dashboard.content.goals import tables
from watcher_dashboard.content.goals import tabs as wtabs
from watcher_dashboard.content.strategies import tables as strategies_tables

LOG = logging.getLogger(__name__)


class IndexView(horizon.tables.DataTableView):
    table_class = tables.GoalsTable
    template_name = 'infra_optim/goals/index.html'
    page_title = _("Goals")

    def get_context_data(self, **kwargs):
        context = super(IndexView, self).get_context_data(**kwargs)
        context['goals_count'] = self.get_goals_count()
        return context

    def get_data(self):
        goals = []
        search_opts = self.get_filters()
        try:
            goals = watcher.Goal.list(self.request, **search_opts)
        except Exception:
            horizon.exceptions.handle(
                self.request,
                _("Unable to retrieve goal information."))
        return goals

    def get_goals_count(self):
        return len(self.get_data())

    def get_filters(self):
        filters = {}
        filter_action = self.table._meta._filter_action
        if filter_action:
            filter_field = self.table.get_filter_field()
            if filter_action.is_api_filter(filter_field):
                filter_string = self.table.get_filter_string()
                if filter_field and filter_string:
                    filters[filter_field] = filter_string
        return filters


class DetailView(horizon.tables.MultiTableView):
    table_classes = (tables.EfficacySpecificationTable,
                     strategies_tables.RelatedStrategiesTable)

    tab_group_class = wtabs.GoalDetailTabs
    template_name = 'infra_optim/goals/details.html'
    redirect_url = 'horizon:admin:goals:index'
    page_title = _("Goal Details: {{ goal.name }}")

    @memoized.memoized_method
    def _get_data(self):
        goal_uuid = None
        try:
            goal_uuid = self.kwargs['goal_uuid']
            goal = watcher.Goal.get(self.request, goal_uuid)
        except Exception as exc:
            LOG.exception(exc)
            msg = _('Unable to retrieve details for goal "%s".') \
                % goal_uuid
            horizon.exceptions.handle(
                self.request, msg,
                redirect=self.redirect_url)
        return goal

    def get_related_strategies_data(self):
        try:
            goal = self._get_data()
            strategies = watcher.Strategy.list(self.request, goal=goal.uuid)
        except Exception as exc:
            LOG.exception(exc)
            strategies = []
            msg = _('Strategy list cannot be retrieved.')
            horizon.exceptions.handle(self.request, msg)

        return strategies

    def get_efficacy_specification_data(self):
        try:
            goal = self._get_data()
            indicators_spec = [watcher.EfficacyIndicatorSpec(spec)
                               for spec in goal.efficacy_specification]
        except Exception as exc:
            LOG.exception(exc)
            indicators_spec = []
            msg = _('Efficacy specification cannot be retrieved.')
            horizon.exceptions.handle(self.request, msg)

        return indicators_spec

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        goal = self._get_data()
        context["goal"] = goal
        return context

    def get_tabs(self, request, *args, **kwargs):
        goal = self._get_data()
        # ports = self._get_ports()
        return self.tab_group_class(request, goal=goal,
                                    # ports=ports,
                                    **kwargs)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import uuid

from openstack_dashboard.test.integration_tests import helpers


class AuditTemplateCreatePanelTests(helpers.AdminTestCase):

    def test_create_audit_template(self):
        """Test the audit template panel:

        * Loads the audit template panel
        * Creates a new audit template with random-generated name
        * Checks that this audit template is in list
        * Deletes this audit template (in tearDown)
        * Checks that the audit template is removed
        """
        audit_template_name = "audit_template_%s" % uuid.uuid1()
        audit_template_page = \
            self.home_pg.go_to_optimization_audittemplatespage()
        audit_template_page.create_audit_template(audit_template_name)
        self.assertTrue(audit_template_page.is_audit_template_present(
            audit_template_name))


class AuditTemplatePanelTests(helpers.AdminTestCase):

    def setUp(self):
        super(AuditTemplatePanelTests, self).setUp()
        self.audit_template_name = "audit_template_%s" % uuid.uuid1()
        audit_template_page = \
            self.home_pg.go_to_optimization_audittemplatespage()
        audit_template_page.create_audit_template(self.audit_template_name)

    def tearDown(self):
        audit_template_page = \
            self.home_pg.go_to_optimization_audittemplatespage()
        audit_template_page.delete_audit_template(self.audit_template_name)
        # Uncomment this line when <Delete> button will be implemented
        self.assertFalse(audit_template_page.is_audit_template_present(
            self.audit_template_name))
        super(AuditTemplatePanelTests, self).tearDown()

    def test_show_audit_template_info(self):
        """Test the audit template panel information page

        * Loads the audit template panel
        * Click on link behind the audit template name
        * Checks the info page (only the "Audit Template Info" title for now)
        """
        audit_template_page = \
            self.home_pg.go_to_optimization_audittemplatespage()
        self.assertTrue(
            audit_template_page.show_audit_template_info(
                self.audit_template_name))

    def test_launch_audit(self):
        """Test the audit template panel "Launch Audit" row button

        * Loads the audit template panel
        * Click on the button "Launch Audit"
        * Checks the audits page for audit template name in page
        """
        audit_template_page = \
            self.home_pg.go_to_optimization_audittemplatespage()
        self.assertTrue(
            audit_template_page.launch_audit(self.audit_template_name))



# Copyright (c) 2016 b<>com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import logging

from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _
import horizon.exceptions
from horizon import forms
import horizon.tables
import horizon.tabs
import horizon.workflows

from watcher_dashboard.api import watcher
from watcher_dashboard.content.audit_templates import forms as wforms
from watcher_dashboard.content.audit_templates import tables
from watcher_dashboard.content.audit_templates import tabs as wtabs

LOG = logging.getLogger(__name__)


class IndexView(horizon.tables.DataTableView):
    table_class = tables.AuditTemplatesTable
    template_name = 'infra_optim/audit_templates/index.html'
    page_title = _("Audit Templates")

    def get_context_data(self, **kwargs):
        context = super(IndexView, self).get_context_data(**kwargs)
        context['audit_templates_count'] = self.get_count()
        return context

    def get_data(self):
        audit_templates = []
        search_opts = self.get_filters()
        try:
            audit_templates = watcher.AuditTemplate.list(
                self.request, **search_opts)
        except Exception as exc:
            LOG.exception(exc)
            horizon.exceptions.handle(
                self.request,
                _("Unable to retrieve audit template information."))
        return audit_templates

    def get_count(self):
        return len(self.get_data())

    def get_filters(self):
        filters = {}
        filter_action = self.table._meta._filter_action
        if filter_action:
            filter_field = self.table.get_filter_field()
            if filter_action.is_api_filter(filter_field):
                filter_string = self.table.get_filter_string()
                if filter_field and filter_string:
                    filters[filter_field] = filter_string
        return filters


class CreateView(forms.ModalFormView):
    form_class = wforms.CreateForm
    form_id = "create_audit_templates_form"
    modal_header = _("Create Audit Template")
    template_name = 'infra_optim/audit_templates/create.html'
    success_url = reverse_lazy("horizon:admin:audit_templates:index")
    page_title = _("Create an Audit Template")
    submit_label = _("Create Audit Template")
    submit_url = reverse_lazy("horizon:admin:audit_templates:create")

    def get_object_id(self, obj):
        return obj.uuid


class DetailView(horizon.tabs.TabbedTableView):
    tab_group_class = wtabs.AuditTemplateDetailTabs
    template_name = 'infra_optim/audit_templates/details.html'
    redirect_url = 'horizon:admin:audit_templates:index'
    page_title = _("Audit Template Details: {{ audit_template.name }}")

    def _get_data(self):
        audit_template_uuid = None
        try:
            LOG.info(self.kwargs)
            audit_template_uuid = self.kwargs['audit_template_uuid']
            audit_template = watcher.AuditTemplate.get(
                self.request, audit_template_uuid)
            if audit_template.scope:
                audit_template.scope = json.dumps(audit_template.scope)

        except Exception as exc:
            LOG.exception(exc)
            msg = _('Unable to retrieve details for audit template "%s".') \
                % audit_template_uuid
            horizon.exceptions.handle(
                self.request, msg,
                redirect=self.redirect_url)
        return audit_template

    def get_related_audits_data(self):
        try:
            audit_template = self._get_data()
            audits = watcher.Audit.list(
                self.request, audit_template=audit_template.uuid)
        except Exception as exc:
            LOG.exception(exc)
            audits = []
            msg = _('Audits list cannot be retrieved.')
            horizon.exceptions.handle(self.request, msg)
        return audits

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        audit_template = self._get_data()
        context["audit_template"] = audit_template
        return context

    def get_tabs(self, request, *args, **kwargs):
        audit_template = self._get_data()
        # ports = self._get_ports()
        return self.tab_group_class(
            request, audit_template=audit_template, **kwargs)



# -*- coding: utf8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import collections

from watcher_dashboard.test import helpers
from watcher_dashboard.utils import utils


class UtilsTests(helpers.TestCase):
    def test_de_camel_case(self):
        ret = utils.de_camel_case('CamelCaseString')
        self.assertEqual(ret, 'Camel Case String')
        ret = utils.de_camel_case('SecureSSLConnection')
        self.assertEqual(ret, 'Secure SSL Connection')
        ret = utils.de_camel_case('xxXXxx')
        self.assertEqual(ret, 'xx X Xxx')
        ret = utils.de_camel_case('XXX')
        self.assertEqual(ret, 'XXX')
        ret = utils.de_camel_case('NON Camel Case')
        self.assertEqual(ret, 'NON Camel Case')

    def test_list_to_dict(self):
        Item = collections.namedtuple('Item', 'id')
        ret = utils.list_to_dict([Item('foo'), Item('bar'), Item('bar')])
        self.assertEqual(ret, {'foo': Item('foo'), 'bar': Item('bar')})

    def test_length(self):
        ret = utils.length(iter([]))
        self.assertEqual(ret, 0)
        ret = utils.length(iter([1, 2, 3]))
        self.assertEqual(ret, 3)

    def test_check_image_type(self):
        Image = collections.namedtuple('Image', 'properties')
        ret = utils.check_image_type(Image({'type': 'Picasso'}), 'Picasso')
        self.assertTrue(ret)
        ret = utils.check_image_type(Image({'type': 'Picasso'}), 'Van Gogh')
        self.assertFalse(ret)
        ret = utils.check_image_type(Image({}), 'Van Gogh')
        self.assertTrue(ret)

    def test_filter_items(self):
        Item = collections.namedtuple('Item', 'index')
        items = [Item(i) for i in range(7)]
        ret = list(utils.filter_items(items, index=1))
        self.assertEqual(ret, [Item(1)])
        ret = list(utils.filter_items(items, index__in=(1, 2, 3)))
        self.assertEqual(ret, [Item(1), Item(2), Item(3)])
        ret = list(utils.filter_items(items, index__not_in=(1, 2, 3)))
        self.assertEqual(ret, [Item(0), Item(4), Item(5), Item(6)])

    def test_safe_int_cast(self):
        ret = utils.safe_int_cast(1)
        self.assertEqual(ret, 1)
        ret = utils.safe_int_cast('1')
        self.assertEqual(ret, 1)
        ret = utils.safe_int_cast('')
        self.assertEqual(ret, 0)
        ret = utils.safe_int_cast(None)
        self.assertEqual(ret, 0)
        ret = utils.safe_int_cast(object())
        self.assertEqual(ret, 0)



# Copyright (c) 2016 b<>com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

from django.template.defaultfilters import title  # noqa
from django import urls
from django.utils.translation import gettext_lazy as _
from django.utils.translation import pgettext_lazy
import horizon.exceptions
import horizon.messages
import horizon.tables
from horizon.utils import filters

from watcher_dashboard.api import watcher

LOG = logging.getLogger(__name__)


ACTION_STATE_DISPLAY_CHOICES = (
    ("NO STATE", pgettext_lazy("Power state of an Instance", u"No State")),
    ("ONGOING", pgettext_lazy("Power state of an Instance", u"On Going")),
    ("SUCCEEDED", pgettext_lazy("Power state of an Instance", u"Succeeded")),
    ("CANCELLED", pgettext_lazy("Power state of an Instance", u"Cancelled")),
    ("FAILED", pgettext_lazy("Power state of an Instance", u"Failed")),
    ("DELETED", pgettext_lazy("Power state of an Instance", u"Deleted")),
    ("PENDING", pgettext_lazy("Power state of an Instance", u"Pending")),
)


class UpdateRow(horizon.tables.Row):
    ajax = True

    def get_data(self, request, action_id):
        action = None
        try:
            action = watcher.Action.get(request, action_id)
        except Exception:
            msg = _('Failed to get the action.')
            LOG.info(msg)
            horizon.messages.warning(request, msg)

        return action


class ActionsFilterAction(horizon.tables.FilterAction):
    filter_type = "server"
    filter_choices = (('action_plan', _("Action Plan ID ="), True),)
    policy_rules = (("infra-optim", "action:detail"),)


def get_action_plan_link(datum):
    try:
        return urls.reverse(
            "horizon:admin:action_plans:detail",
            kwargs={"action_plan_uuid": getattr(
                datum, "action_plan_uuid", None)})
    except urls.NoReverseMatch:
        return None


class ActionsTable(horizon.tables.DataTable):
    name = horizon.tables.Column(
        'uuid',
        verbose_name=_("UUID"),
        link="horizon:admin:actions:detail")
    action_type = horizon.tables.Column(
        'action_type',
        verbose_name=_('Type'),
        filters=(title, filters.replace_underscores))
    state = horizon.tables.Column(
        'state',
        verbose_name=_('State'),
        status_choices=ACTION_STATE_DISPLAY_CHOICES)
    action_plan = horizon.tables.Column(
        'action_plan_uuid',
        verbose_name=_('Action Plan'),
        link=get_action_plan_link)

    def get_object_id(self, datum):
        return datum.uuid

    class Meta(object):
        name = "wactions"
        verbose_name = _("Actions")
        table_actions = (ActionsFilterAction, )
        row_class = UpdateRow


class RelatedActionsTable(horizon.tables.DataTable):
    """Identical to the index table but with different Meta"""
    name = horizon.tables.Column(
        'uuid',
        verbose_name=_("UUID"),
        link="horizon:admin:actions:detail")
    action_type = horizon.tables.Column(
        'action_type',
        verbose_name=_('Type'),
        filters=(title, filters.replace_underscores))
    state = horizon.tables.Column(
        'state',
        verbose_name=_('State'),
        status_choices=ACTION_STATE_DISPLAY_CHOICES)
    action_plan = horizon.tables.Column(
        'action_plan_uuid',
        verbose_name=_('Action Plan'),
        link=get_action_plan_link)

    def get_object_id(self, datum):
        return datum.uuid

    class Meta(object):
        name = "related_wactions"
        verbose_name = _("Related Actions")
        hidden_title = False


class ActionParametersTable(horizon.tables.DataTable):
    name = horizon.tables.Column(
        'name',
        verbose_name=_("Parameter name"))
    value = horizon.tables.Column(
        'value',
        verbose_name=_('Parameter value'))

    def get_object_id(self, datum):
        return datum.name

    class Meta(object):
        name = "parameters"
        verbose_name = _("Related parameters")
        hidden_title = False



# Copyright (c) 2016 b<>com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.utils.translation import gettext_lazy as _
from horizon import tabs


class OverviewTab(tabs.Tab):
    name = _("Overview")
    slug = "overview"
    template_name = "infra_optim/audits/_detail_overview.html"

    def get_context_data(self, request):
        return {"audit": self.tab_group.kwargs['audit']}


class AuditDetailTabs(tabs.TabGroup):
    slug = "audit_details"
    tabs = (OverviewTab,)
    sticky = True



# Copyright 2012,  Nachi Ueno,  NTT MCL,  Inc.
# All rights reserved.

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
Forms for starting Watcher Audit Templates.
"""
import logging

from django.core import exceptions as core_exc
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from horizon import exceptions
from horizon import forms
from horizon import messages
import yaml

from watcher_dashboard.api import watcher

LOG = logging.getLogger(__name__)


class YamlValidator(object):
    message = _('Enter a valid YAML or JSON value.')
    code = 'invalid'

    def __init__(self, message=None):
        if message:
            self.message = message

    def __call__(self, value):
        try:
            yaml.safe_load(value)
        except Exception:
            raise core_exc.ValidationError(self.message, code=self.code)


class CreateForm(forms.SelfHandlingForm):
    name = forms.CharField(max_length=255, label=_("Name"))
    description = forms.CharField(max_length=255, label=_("Description"),
                                  required=False)
    goal = forms.ChoiceField(label=_('Goal'))
    strategy = forms.DynamicChoiceField(label=_('Strategy'), required=False)

    scope = forms.CharField(
        label=_('Scope'), required=False,
        widget=forms.widgets.Textarea,
        validators=[YamlValidator()])

    failure_url = 'horizon:admin:audit_templates:index'

    def __init__(self, request, *args, **kwargs):
        super(CreateForm, self).__init__(request, *args, **kwargs)
        goals = self._get_goal_list(request)
        strategies = self._get_strategy_list(request, goals)

        if goals:
            self.fields['goal'].choices = goals
        else:
            del self.fields['goal']

        if strategies:
            self.fields['strategy'].choices = strategies
        else:
            del self.fields['strategy']

    def _get_goal_list(self, request):
        try:
            goals = watcher.Goal.list(self.request)
        except Exception as exc:
            msg = _('Failed to get goals list: %s') % str(exc)
            LOG.info(msg)
            messages.warning(request, msg)
            messages.warning(request, exc)
            goals = []

        choices = [
            (goal.uuid, goal.display_name)
            for goal in goals
        ]

        if choices:
            choices.insert(0, ("", _("Select Goal")))
        return choices

    def _get_strategy_list(self, request, goals):
        try:
            strategies = watcher.Strategy.list(self.request)
        except Exception as exc:
            msg = _('Failed to get the list of available strategies.')
            LOG.info(msg)
            messages.warning(request, msg)
            messages.warning(request, exc)
            strategies = []

        _goals = {}
        for goal in goals:
            _goals[goal[0]] = goal[1]

        choices = [
            (strategy.uuid, strategy.display_name +
             ' (GOAL: ' + _goals[strategy.goal_uuid] + ')')
            for strategy in strategies
        ]

        if choices:
            choices.insert(0, ("", _("Select Strategy")))
        return choices

    def handle(self, request, data):
        try:
            params = {'name': data['name']}
            params['description'] = data['description']
            params['goal'] = data['goal']
            params['strategy'] = data['strategy'] or None
            params['scope'] = [] if not data['scope'] else yaml.safe_load(
                data['scope'])
            audit_tpl = watcher.AuditTemplate.create(request, **params)
            message = _('Audit Template was successfully created.')
            messages.success(request, message)
            return audit_tpl
        except Exception as exc:
            msg = _('Failed to create audit template.')
            LOG.info(exc)
            redirect = reverse(self.failure_url)
            exceptions.handle(request, msg, redirect=redirect)
            return False



# Copyright (c) 2016 b<>com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from django import urls

from watcher_dashboard import api
from watcher_dashboard.test import helpers as test

INDEX_URL = urls.reverse(
    'horizon:admin:audit_templates:index')
CREATE_URL = urls.reverse(
    'horizon:admin:audit_templates:create')
DETAILS_VIEW = 'horizon:admin:audit_templates:detail'


class AuditTemplatesTest(test.BaseAdminViewTests):

    def setUp(self):
        super(AuditTemplatesTest, self).setUp()
        self.goal_list = self.goals.list()
        self.strategy_list = self.strategies.list()

    @mock.patch.object(api.watcher.AuditTemplate, 'list')
    def test_index(self, mock_list):
        mock_list.return_value = self.audit_templates.list()

        res = self.client.get(INDEX_URL)
        self.assertTemplateUsed(res, 'infra_optim/audit_templates/index.html')
        audit_templates = res.context['audit_templates_table'].data
        self.assertCountEqual(audit_templates, self.audit_templates.list())

    @mock.patch.object(api.watcher.AuditTemplate, 'list')
    def test_audit_template_list_unavailable(self, mock_list):
        mock_list.side_effect = self.exceptions.watcher

        resp = self.client.get(INDEX_URL)
        self.assertMessageCount(resp, error=1, warning=0)

    @mock.patch.object(api.watcher.Strategy, 'list')
    @mock.patch.object(api.watcher.Goal, 'list')
    def test_create_get(self, m_goal_list, m_strategy_list):
        m_goal_list.return_value = self.goal_list
        m_strategy_list.return_value = self.strategy_list
        res = self.client.get(CREATE_URL)
        self.assertTemplateUsed(res, 'infra_optim/audit_templates/create.html')

    @mock.patch.object(api.watcher.Strategy, 'list')
    @mock.patch.object(api.watcher.Goal, 'list')
    @mock.patch.object(api.watcher.AuditTemplate, 'create')
    def test_create_post(self, m_audit_create, m_goal_list, m_strategy_list):
        at = self.audit_templates.first()
        form_data = {
            'name': at.name,
            'goal': at.goal_uuid,
            'strategy': at.strategy_uuid,
            'description': at.description,
            'scope': at.scope,
        }
        m_goal_list.return_value = self.goal_list
        m_strategy_list.return_value = self.strategy_list
        m_audit_create.return_value = at

        res = self.client.post(CREATE_URL, form_data)
        self.assertNoFormErrors(res)
        self.assertRedirectsNoFollow(res, INDEX_URL)

    @mock.patch.object(api.watcher.AuditTemplate, 'get')
    def test_details(self, m_get):
        at = self.audit_templates.first()
        at_id = at.uuid
        m_get.return_value = at

        DETAILS_URL = urls.reverse(DETAILS_VIEW, args=[at_id])
        res = self.client.get(DETAILS_URL)
        self.assertTemplateUsed(res,
                                'infra_optim/audit_templates/details.html')
        audit_templates = res.context['audit_template']
        self.assertCountEqual([audit_templates], [at])

    @mock.patch.object(api.watcher.AuditTemplate, 'get')
    def test_details_exception(self, m_get):
        at = self.audit_templates.first()
        at_id = at.uuid
        m_get.side_effect = self.exceptions.watcher

        DETAILS_URL = urls.reverse(DETAILS_VIEW, args=[at_id])
        res = self.client.get(DETAILS_URL)
        self.assertRedirectsNoFollow(res, INDEX_URL)

    @mock.patch.object(api.watcher.AuditTemplate, 'delete')
    @mock.patch.object(api.watcher.AuditTemplate, 'list')
    def test_delete(self, m_list, m_del):
        at_list = self.audit_templates.list()
        at = self.audit_templates.first()
        at_id = at.uuid
        m_list.return_value = at_list

        form_data = {'action': 'audit_templates__delete',
                     'object_ids': at_id}

        res = self.client.post(INDEX_URL, form_data)
        self.assertRedirectsNoFollow(res, INDEX_URL)



# Copyright (c) 2016 b<>com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

from django.utils.translation import gettext_lazy as _
import horizon.exceptions
import horizon.tables
import horizon.tabs
from horizon.utils import memoized
import horizon.workflows

from watcher_dashboard.api import watcher
from watcher_dashboard.content.strategies import tables
from watcher_dashboard.content.strategies import tabs as wtabs

LOG = logging.getLogger(__name__)


class IndexView(horizon.tables.DataTableView):
    table_class = tables.StrategiesTable
    template_name = 'infra_optim/strategies/index.html'
    page_title = _("Strategies")

    def get_context_data(self, **kwargs):
        context = super(IndexView, self).get_context_data(**kwargs)
        context['strategies_count'] = self.get_strategies_count()
        return context

    def get_data(self):
        strategies = []
        search_opts = self.get_filters()
        try:
            strategies = watcher.Strategy.list(self.request, **search_opts)
        except Exception as exc:
            LOG.exception(exc)
            horizon.exceptions.handle(
                self.request,
                _("Unable to retrieve strategy information."))
        return strategies

    def get_strategies_count(self):
        return len(self.get_data())

    def get_filters(self):
        filters = {}
        filter_action = self.table._meta._filter_action
        if filter_action:
            filter_field = self.table.get_filter_field()
            if filter_action.is_api_filter(filter_field):
                filter_string = self.table.get_filter_string()
                if filter_field and filter_string:
                    filters[filter_field] = filter_string
        return filters


class DetailView(horizon.tabs.TabbedTableView):
    tab_group_class = wtabs.StrategyDetailTabs
    template_name = 'infra_optim/strategies/details.html'
    redirect_url = 'horizon:admin:strategies:index'
    page_title = _("Strategy Details: {{ strategy.name }}")

    @memoized.memoized_method
    def _get_data(self):
        strategy_uuid = None
        try:
            strategy_uuid = self.kwargs['strategy_uuid']
            strategy = watcher.Strategy.get(self.request, strategy_uuid)
        except Exception as exc:
            LOG.exception(exc)
            msg = _('Unable to retrieve details for strategy "%s".') \
                % strategy_uuid
            horizon.exceptions.handle(
                self.request, msg,
                redirect=self.redirect_url)
        return strategy

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        strategy = self._get_data()
        context["strategy"] = strategy
        return context

    def get_tabs(self, request, *args, **kwargs):
        strategy = self._get_data()
        return self.tab_group_class(request, strategy=strategy, **kwargs)



# -*- coding: utf8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import re

CAMEL_RE = re.compile(r'([A-Z][a-z]+|[A-Z]+(?=[A-Z\s]|$))')


def de_camel_case(text):
    """Convert CamelCase names to human-readable format."""
    return ' '.join(w.strip() for w in CAMEL_RE.split(text) if w.strip())


def list_to_dict(object_list, key_attribute='id'):
    """Converts an object list to a dict

    :param object_list: list of objects to be put into a dict
    :type  object_list: list

    :param key_attribute: object attribute used as index by dict
    :type  key_attribute: str

    :return: dict containing the objects in the list
    :rtype: dict
    """
    return dict((getattr(o, key_attribute), o) for o in object_list)


def length(iterator):
    """A length function for iterators

    Returns the number of items in the specified iterator. Note that this
    function consumes the iterator in the process.
    """
    return sum(1 for _item in iterator)


def check_image_type(image, image_type):
    """Check if image 'type' property matches passed-in image_type.

    If image has no 'type' property' return True, as we cannot
    be sure what type of image it is.
    """

    return (image.properties.get('type', image_type) == image_type)


def filter_items(items, **kwargs):
    """Filters the list of items and returns the filtered list.

    Example usage:
    >>> class Item(object):
    ...     def __init__(self, index):
    ...         self.index = index
    ...     def __repr__(self):
    ...         return '<Item index=%d>' % self.index
    >>> items = [Item(i) for i in range(7)]
    >>> list(filter_items(items, index=1))
    [<Item index=1>]
    >>> list(filter_items(items, index__in=(1, 2, 3)))
    [<Item index=1>, <Item index=2>, <Item index=3>]
    >>> list(filter_items(items, index__not_in=(1, 2, 3)))
    [<Item index=0>, <Item index=4>, <Item index=5>, <Item index=6>]
    """
    for item in items:
        for name, value in kwargs.items():
            if name.endswith('__in'):
                if getattr(item, name[:-len('__in')]) not in value:
                    break
            elif name.endswith('__not_in'):
                if getattr(item, name[:-len('__not_in')]) in value:
                    break
            else:
                if getattr(item, name) != value:
                    break
        else:
            yield item


def safe_int_cast(value):
    try:
        return int(value)
    except (TypeError, ValueError):
        return 0



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

from django.conf import settings
from django.utils.translation import gettext_lazy as _
from openstack_dashboard.api import base
from watcherclient import client as wc

from watcher_dashboard.utils import errors as errors_utils

LOG = logging.getLogger(__name__)
WATCHER_SERVICE = 'infra-optim'


def watcherclient(request, password=None):
    api_version = "1"
    insecure = getattr(settings, 'OPENSTACK_SSL_NO_VERIFY', False)
    ca_file = getattr(settings, 'OPENSTACK_SSL_CACERT', None)
    insert_watcher_policy_file()

    endpoint = base.url_for(request, WATCHER_SERVICE)

    LOG.debug('watcherclient connection created using token "%s" and url "%s"'
              % (request.user.token.id, endpoint))

    client = wc.get_client(
        api_version,
        watcher_url=endpoint,
        insecure=insecure,
        ca_file=ca_file,
        username=request.user.username,
        password=password,
        os_auth_token=request.user.token.id
    )
    return client


def insert_watcher_policy_file():
    policy_files = getattr(settings, 'POLICY_FILES', {})
    policy_files['infra-optim'] = 'watcher_policy.json'
    setattr(settings, 'POLICY_FILES', policy_files)


class Audit(base.APIDictWrapper):
    _attrs = ('uuid', 'name', 'created_at', 'modified_at', 'deleted_at',
              'state', 'audit_type', 'audit_template_uuid',
              'audit_template_name', 'interval')

    def __init__(self, apiresource, request=None):
        super(Audit, self).__init__(apiresource)
        self._request = request

    @classmethod
    def create(cls, request, audit_template_uuid, audit_type, name=None,
               auto_trigger=False, interval=None):

        """Create an audit in Watcher

        :param request: request object
        :type  request: django.http.HttpRequest

        :param audit_template_uuid: related audit template UUID
        :type  audit_template_uuid: string

        :param audit_type: audit type
        :type  audit_type: string

        :param interval: Audit interval (default: None)
        :type  interval: int

        :param name: Name for this audit
        :type  name: string

        :return: the created Audit object
        :rtype:  :py:class:`~.Audit`
        """

        if interval:
            return watcherclient(request).audit.create(
                audit_template_uuid=audit_template_uuid, audit_type=audit_type,
                auto_trigger=auto_trigger, interval=interval, name=name)
        else:
            return watcherclient(request).audit.create(
                audit_template_uuid=audit_template_uuid, audit_type=audit_type,
                auto_trigger=auto_trigger, name=name)

    @classmethod
    def list(cls, request, **filters):
        """Return a list of audits in Watcher

        :param request: request object
        :type  request: django.http.HttpRequest

        :param filters: key/value kwargs used as filters
        :type  filters: dict

        :return: list of audits, or an empty list if there are none
        :rtype:  list of :py:class:`~.Audit`
        """
        return watcherclient(request).audit.list(detail=True, **filters)

    @classmethod
    @errors_utils.handle_errors(_("Unable to retrieve audit"))
    def get(cls, request, audit_id):
        """Return the audit that matches the ID

        :param request: request object
        :type  request: django.http.HttpRequest

        :param audit_id: id of audit to be retrieved
        :type  audit_id: int

        :return: matching audit, or None if no audit matches
                 the ID
        :rtype:  :py:class:`~.Audit`
        """
        return watcherclient(request).audit.get(audit=audit_id)

    @classmethod
    def delete(cls, request, audit_id):
        """Delete an audit

        :param request: request object
        :type  request: django.http.HttpRequest

        :param audit_id: audit id
        :type  audit_id: int
        """
        return watcherclient(request).audit.delete(audit=audit_id)

    @classmethod
    def cancel(cls, request, audit_id):
        """Cancel an audit

        :param request: request object
        :type  request: django.http.HttpRequest

        :param audit_id: audit id
        :type  audit_id: int
        """
        cancel_patch = [{'op': 'replace', 'path': '/state',
                         'value': 'CANCELLED'}]
        return watcherclient(request).audit.update(audit=audit_id,
                                                   patch=cancel_patch)

    @property
    def id(self):
        return self.uuid


class AuditTemplate(base.APIDictWrapper):
    _attrs = ('uuid', 'description', 'scope', 'name', 'goal_uuid', 'goal_name',
              'strategy_uuid', 'strategy_name', 'created_at', 'updated_at',
              'deleted_at')

    def __init__(self, apiresource, request=None):
        super(AuditTemplate, self).__init__(apiresource)
        self._request = request

    @classmethod
    def create(cls, request, name, goal, strategy,
               description, scope):
        """Create an audit template in Watcher

        :param request: request object
        :type  request: django.http.HttpRequest

        :param name: Name for this audit template
        :type  name: string

        :param goal: Goal UUID or name associated to this audit template
        :type  goal: string

        :param strategy: Strategy UUID or name associated to this audit
                         template
        :type  strategy: string

        :param description: Descrition of the audit template
        :type  description: string

        :param scope: Audit scope
        :type  scope: list of list of dict

        :param audit_template: audit template
        :type  audit_template: string

        :return: the created Audit Template object
        :rtype:  :py:class:`~.AuditTemplate`
        """
        audit_template = watcherclient(request).audit_template.create(
            name=name,
            goal=goal,
            strategy=strategy,
            description=description,
            scope=scope,
        )

        return audit_template

    @classmethod
    def patch(cls, request, audit_template_id, parameters):
        """Update an audit in Watcher

        :param request: request object
        :type  request: django.http.HttpRequest

        :param audit_template_id: id of the audit template we want to update
        :type  audit_template_id: string

        :param parameters: new values for the audit template's parameters
        :type  parameters: dict

        :return: the updated Audit Template object
        :rtype:  :py:class:`~.AuditTemplate`
        """
        parameter_list = [{
            'name': str(name),
            'value': str(value),
        } for (name, value) in parameters.items()]
        audit_template = watcherclient(request).audit_template.patch(
            audit_template_id, parameter_list)
        return audit_template

    @classmethod
    def list(cls, request, **filters):
        """Return a list of audit templates in Watcher

        :param request: request object
        :type  request: django.http.HttpRequest

        :param filters: key/value kwargs used as filters
        :type  filters: dict

        :return: list of audit templates, or an empty list if there are none
        :rtype:  list of :py:class:`~.AuditTemplate`
        """
        return watcherclient(request).audit_template.list(
            detail=True, **filters)

    @classmethod
    @errors_utils.handle_errors(_("Unable to retrieve audit template"))
    def get(cls, request, audit_template_id):
        """Return the audit template that matches the ID

        :param request: request object
        :type  request: django.http.HttpRequest

        :param audit_template_id: id of audit template to be retrieved
        :type  audit_template_id: int

        :return: matching audit template, or None if no audit template matches
                 the ID
        :rtype:  :py:class:`~.AuditTemplate`
        """
        return watcherclient(request).audit_template.get(
            audit_template_id=audit_template_id)

    @classmethod
    def delete(cls, request, audit_template_id):
        """Delete an audit_template

        :param request: request object
        :type  request: django.http.HttpRequest

        :param audit_template_id: audit id
        :type  audit_template_id: int
        """
        watcherclient(request).audit_template.delete(
            audit_template_id=audit_template_id)

    @property
    def id(self):
        return self.uuid


class ActionPlan(base.APIDictWrapper):
    _attrs = ('uuid', 'created_at', 'updated_at', 'deleted_at',
              'audit_uuid', 'state')

    def __init__(self, apiresource, request=None):
        super(ActionPlan, self).__init__(apiresource)
        self._request = request

    @classmethod
    def list(cls, request, **filters):
        """Return a list of action plans in Watcher

        :param request: request object
        :type  request: django.http.HttpRequest

        :param filters: key/value kwargs used as filters
        :type  filters: dict

        :return: list of action plans, or an empty list if there are none
        :rtype:  list of :py:class:`~.ActionPlan`
        """
        return watcherclient(request).action_plan.list(detail=True, **filters)

    @classmethod
    @errors_utils.handle_errors(_("Unable to retrieve action plan"))
    def get(cls, request, action_plan_id):
        """Return the action plan that matches the ID

        :param request: request object
        :type  request: django.http.HttpRequest

        :param action_plan_id: id of action plan to be retrieved
        :type  action_plan_id: int

        :return: matching action plan, or None if no action plan matches
                 the ID
        :rtype:  :py:class:`~.ActionPlan`
        """
        return watcherclient(request).action_plan.get(
            action_plan_id=action_plan_id)

    @classmethod
    def delete(cls, request, action_plan_id):
        """Delete an action plan

        :param request: request object
        :type  request: django.http.HttpRequest

        :param action_plan_id: audit id
        :type  action_plan_id: int
        """
        watcherclient(request).action_plan.delete(
            action_plan_id=action_plan_id)

    @classmethod
    def start(cls, request, action_plan_id):
        """Start an Action Plan

        :param request: request object
        :type  request: django.http.HttpRequest

        :param action_plan_id: audit id
        :type  action_plan_id: int
        """
        watcherclient(request).action_plan.start(action_plan_id)

    @property
    def id(self):
        return self.uuid


class Action(base.APIDictWrapper):
    _attrs = ('uuid', 'created_at', 'updated_at', 'deleted_at', 'next_uuid',
              'description', 'state', 'action_plan_uuid',
              'action_type', 'applies_to', 'src', 'dst', 'parameter')

    def __init__(self, apiresource, request=None):
        super(Action, self).__init__(apiresource)
        self._request = request

    @classmethod
    def list(cls, request, **filters):
        """Return a list of actions in Watcher

        :param request: request object
        :type  request: django.http.HttpRequest

        :param filters: key/value kwargs used as filters
        :type  filters: dict

        :return: list of actions, or an empty list if there are none
        :rtype:  list of :py:class:`~.Action`
        """
        return watcherclient(request).action.list(detail=True, **filters)

    @classmethod
    @errors_utils.handle_errors(_("Unable to retrieve action"))
    def get(cls, request, action_id):
        """Return the action that matches the ID

        :param request: request object
        :type  request: django.http.HttpRequest

        :param action_id: id of action to be retrieved
        :type  action_id: int

        :return: matching action, or None if no action matches
                 the ID
        :rtype:  :py:class:`~.Action`
        """
        return watcherclient(request).action.get(action_id=action_id)

    @classmethod
    def delete(cls, request, action_id):
        """Delete an action

        :param request: request object
        :type  request: django.http.HttpRequest

        :param action_id: action_plan id
        :type  action_id: int
        """
        watcherclient(request).action.delete(
            action_id=action_id)

    @classmethod
    def start(cls, request, action_id):
        """Start an Action Plan

        :param request: request object
        :type  request: django.http.HttpRequest

        :param action_id: action_plan id
        :type  action_id: int
        """
        patch = []
        patch.append({'op': 'replace', 'path': '/state', 'value': 'PENDING'})
        watcherclient(request).action.update(action_id, patch)

    @property
    def id(self):
        return self.uuid


class Goal(base.APIDictWrapper):
    """Goal resource."""

    _attrs = ('uuid', 'name', 'display_name', 'created_at',
              'updated_at', 'deleted_at', 'efficacy_specifications')

    def __init__(self, apiresource, request=None):
        super(Goal, self).__init__(apiresource)
        self._request = request

    @classmethod
    def list(cls, request, **filters):
        """Return a list of goals in Watcher

        :param request: request object
        :type  request: django.http.HttpRequest

        :param filters: key/value kwargs used as filters
        :type  filters: dict

        :return: list of goals, or an empty list if there are none
        :rtype:  list of :py:class:`~.Goal` instance
        """
        return watcherclient(request).goal.list(detail=True, **filters)

    @classmethod
    @errors_utils.handle_errors(_("Unable to retrieve goal"))
    def get(cls, request, goal):
        """Return the goal that matches the ID

        :param request: request object
        :type  request: django.http.HttpRequest

        :param goal: uuid of goal to be retrieved
        :type  goal: int

        :return: matching goal, or None if no goal matches the UUID
        :rtype:  :py:class:`~.Goal` instance
        """
        return watcherclient(request).goal.get(goal)

    @property
    def id(self):
        return self.uuid


class Strategy(base.APIDictWrapper):
    """Strategy resource."""

    _attrs = ('uuid', 'name', 'display_name', 'goal_uuid', 'goal_name',
              'created_at', 'updated_at', 'deleted_at')

    def __init__(self, apiresource, request=None):
        super(Strategy, self).__init__(apiresource)
        self._request = request

    @classmethod
    def list(cls, request, **filters):
        """Return a list of strategies in Watcher

        :param request: request object
        :type  request: django.http.HttpRequest

        :param filters: key/value kwargs used as filters
        :type  filters: dict

        :return: list of strategies, or an empty list if there are none
        :rtype:  list of :py:class:`~.Strategy` instances
        """
        return watcherclient(request).strategy.list(detail=True, **filters)

    @classmethod
    @errors_utils.handle_errors(_("Unable to retrieve strategy"))
    def get(cls, request, strategy):
        """Return the strategy that matches the UUID

        :param request: request object
        :type  request: django.http.HttpRequest

        :param strategy: uuid of strategy to be retrieved
        :type  strategy: str

        :return: matching strategy, or None if no strategy matches the UUID
        :rtype:  :py:class:`~.Strategy` instance
        """
        return watcherclient(request).strategy.get(strategy)

    @property
    def id(self):
        return self.uuid


class EfficacyIndicatorSpec(base.APIDictWrapper):

    attrs = ('name', 'description', 'unit', 'schema')


class EfficacyIndicator(base.APIDictWrapper):

    def __init__(self, indicator):
        super(EfficacyIndicator, self).__init__(indicator)
        self.value = indicator.get('value', None)
        self.name = indicator.get('name', None)
        self.description = indicator.get('description', None)
        self.unit = indicator.get('unit', None)



# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
# Copyright 2010 OpenStack Foundation
# Copyright 2013 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import sys

import install_venv_common as install_venv  # noqa


def print_help(venv, root):
    help = """
    OpenStack development environment setup is complete.

    OpenStack development uses virtualenv to track and manage Python
    dependencies while in development and testing.

    To activate the OpenStack virtualenv for the extent of your current shell
    session you can run:

    $ . %s/bin/activate

    Or, if you prefer, you can run commands in the virtualenv on a case by case
    basis by running:

    $ %s/tools/with_venv.sh <your command>

    Also, make test will automatically use the virtualenv.
    """
    print(help % (venv, root))


def main(argv):
    root = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))

    if os.environ.get('tools_path'):
        root = os.environ['tools_path']
    venv = os.path.join(root, '.venv')
    if os.environ.get('venv'):
        venv = os.environ['venv']

    pip_requires = os.path.join(root, 'requirements.txt')
    test_requires = os.path.join(root, 'test-requirements.txt')
    py_version = "python%s.%s" % (sys.version_info[0], sys.version_info[1])
    project = 'OpenStack'
    install = install_venv.InstallVenv(root, venv, pip_requires, test_requires,
                                       py_version, project)
    options = install.parse_args(argv)
    install.check_python_version()
    install.check_dependencies()
    install.create_virtualenv(no_site_packages=options.no_site_packages)
    install.install_dependencies()
    print_help(venv, root)

if __name__ == '__main__':
    main(sys.argv)
